@@ -1,4 +1,4 @@
-/*  $Id: non-string-nodes.cpp,v 1.1 2005/02/03 23:28:34 sarrazip Exp $
+/*  $Id: non-string-nodes.cpp,v 1.2 2010/06/08 04:36:47 sarrazip Exp $
     Example where the expression tree nodes are not based on strings.
 
     Written by Pierre Sarrazin <http://sarrazip.com/>
@@ -37,7 +37,7 @@ bool operator < (Node a, Node b)
 }
 
 
-int main(int argc, char *argv[])
+int main(int /* argc */, char * /* argv */ [])
 {
     {
 	try
