@@ -1,4 +1,4 @@
-/*  $Id: BoolExpr.cpp,v 1.29 2013/03/03 08:19:27 sarrazip Exp $
+/*  $Id: BoolExpr.cpp,v 1.31 2014/03/01 17:30:58 sarrazip Exp $
     BoolExpr.cpp - Boolean expression binary tree node
 
     boolstuff - Disjunctive Normal Form boolean expression library
@@ -333,6 +333,7 @@ BoolExpr<T>::print(std::ostream &out) const
 
 	case NOT:
 	    {
+                assert(right != NULL);
 		bool paren = (right->type == AND || right->type == OR);
 		out << '!';
 		if (paren)
@@ -345,6 +346,8 @@ BoolExpr<T>::print(std::ostream &out) const
 
 	case OR:
 	    {
+                assert(left != NULL);
+                assert(right != NULL);
 		left->print(out);
 		out << "|";
 		right->print(out);
@@ -353,6 +356,8 @@ BoolExpr<T>::print(std::ostream &out) const
 
 	case AND:
 	    {
+                assert(left != NULL);
+                assert(right != NULL);
 		bool paren = (left->type == OR);
 		if (paren)
 		    out << '(';
@@ -475,6 +480,10 @@ BoolExpr<T>::getDisjunctiveNormalForm(BoolExpr<T> *root)
     std::vector<BoolExpr<T> *> termRoots;
     dnfRoot->getDNFTermRoots(std::inserter(termRoots, termRoots.end()));
 
+    // Destroy the OR nodes.
+    destroyDNFBinaryOpNodes(dnfRoot, true);
+    dnfRoot = NULL;  // dnfRoot is invalid at this point
+
     simplifyConjunctionList(termRoots);
 
     // If only one term, consider it "useful",
@@ -509,30 +518,26 @@ BoolExpr<T>::getDisjunctiveNormalForm(BoolExpr<T> *root)
     }
     assert(isUsefulTerm.size() == termRoots.size());
 
-    // Nothing to do if all the terms are useful:
-    if (usefulTerms.size() == termRoots.size())
-	return dnfRoot;
-
-    destroyDNFBinaryOpNodes(dnfRoot, true);
-    // dnfRoot now invalid; termRoots holds the term trees
-
-    // If all the terms are useless (e.g., a&!a | b&!b),
-    // return the first one, because we must return something.
-    if (usefulTerms.size() == 0)
+    if (usefulTerms.size() != termRoots.size())  // if not all terms useful
     {
-	// Destroy all term trees except the first one.
-	for (it = termRoots.begin(), it++; it != termRoots.end(); it++)
-	    delete *it;
+        // If all the terms are useless (e.g., a&!a | b&!b),
+        // return the first one, because we must return something.
+        if (usefulTerms.size() == 0)
+        {
+            // Destroy all term trees except the first one.
+            for (it = termRoots.begin(), it++; it != termRoots.end(); it++)
+                delete *it;
 
-	// Return the first term.
-	return termRoots.front();
+            // Return the first term.
+            return termRoots.front();
+        }
+
+        // Destroy the trees in 'termRoots' that are not useful:
+        for (it = termRoots.begin(); it != termRoots.end(); it++)
+            if (!isUsefulTerm[it - termRoots.begin()])
+                delete *it;
     }
 
-    // Destroy the trees in 'termRoots' that are not useful:
-    for (it = termRoots.begin(); it != termRoots.end(); it++)
-	if (!isUsefulTerm[it - termRoots.begin()])
-	    delete *it;
-
     // Join the useful terms with OR nodes:
     return joinTreesWithOrNodes(usefulTerms);
 }
@@ -549,6 +554,14 @@ BoolExpr<T>::simplifyConjunctionList(std::vector<BoolExpr<T> *> &conjunctionList
 }
 
 
+// Assumes that 'conj' is a term of a DNF, i.e., there are no OR nodes.
+// Gets the factor roots from 'conj'.
+// For example, if 'conj' represents a&b&c, those roots are a, b and c.
+// The AND nodes are destroyed.
+// Then the factor roots are sorted and duplicates are destroyed.
+// A new conjunction is formed from the remaining factor roots.
+// If a single factor root is left, it is returned as is.
+//
 template <class T>
 /*static*/
 BoolExpr<T> *
@@ -562,6 +575,10 @@ BoolExpr<T>::simplifyConjunction(BoolExpr<T> *conj)
     std::vector<BoolExpr<T> *> factorRoots;
     conj->getDNFFactorRoots(std::inserter(factorRoots, factorRoots.end()));
 
+    // Destroy the AND nodes.
+    destroyDNFBinaryOpNodes(conj, false);
+    conj = NULL;  // conj now invalid; factorRoots holds the term trees
+
     // Sort the factors to find the duplicates.
     //
     std::sort(factorRoots.begin(), factorRoots.end(), exprOrdering<T>);
@@ -586,9 +603,6 @@ BoolExpr<T>::simplifyConjunction(BoolExpr<T> *conj)
             previous = f;
     }
 
-    destroyDNFBinaryOpNodes(conj, false);
-    // conj now invalid; factorRoots holds the term trees
-
     // Build new conjunction from non null pointers in factorRoots[].
     BoolExpr<T> *newConj = NULL;
     for (it = factorRoots.begin(); it != factorRoots.end(); it++)
@@ -1063,13 +1077,22 @@ BoolExpr<T>::negateDNF(BoolExpr<T> *root, bool &tooLarge)
     typename std::vector<const BoolExpr<T> *>::const_iterator it;
     size_t product = 1;  // will be the product of the sizes of the terms
     const size_t max = 10000;  // maximum allowed product (to avoid using excessive memory)
+    size_t indexOfFirstUselessTerm = size_t(-1);
     for (it = termRoots.begin(); it != termRoots.end(); it++)
     {
 	std::set<T> pos, neg;
 	(*it)->getTreeVariables(pos, neg);
 	bool useful = (*it)->isDNFTermUseful(pos, neg);
-	if (useful)
+	if (useful || indexOfFirstUselessTerm == size_t(-1))
 	{
+	    if (!useful)
+	    {
+            // Remember position of 1st useless term encountered.
+            // If all terms are useless, we will use it, because
+            // we must return something.
+            indexOfFirstUselessTerm = terms.size();
+	    }
+
 	    terms.push_back(std::vector< Literal<T> >());
 	    std::vector< Literal<T> > &variables = terms.back();
 
@@ -1098,7 +1121,18 @@ BoolExpr<T>::negateDNF(BoolExpr<T> *root, bool &tooLarge)
     delete root;
     root = NULL;
 
+    // If at least one useless term has been seen, but a useful term
+    // has also been seen, then get rid of the useless term designated
+    // by 'indexOfFirstUselessTerm'.
+    if (indexOfFirstUselessTerm != size_t(-1) && terms.size() > 1)
+    {
+        terms.erase(terms.begin() + indexOfFirstUselessTerm);
+    }
 
+
+    // Create a disjunction that is the negation of the useful terms
+    // seen in the original expression.
+    //
     std::vector<size_t> indexVec(terms.size(), 0);
     size_t numTermsCreated = 0;
     BoolExpr<T> *disjunction = NULL;
@@ -1138,16 +1172,13 @@ BoolExpr<T>::negateDNF(BoolExpr<T> *root, bool &tooLarge)
 	    disjunction = new BoolExpr<T>(OR, disjunction, conjunction);
 
 	size_t i = indexVec.size() - 1;
-	for (;;)
+	for ( ; i != size_t(-1); --i)
 	{
 	    indexVec[i]++;
 	    if (indexVec[i] < terms[i].size())
 		break;
 	    // Overflow:
 	    indexVec[i] = 0;
-	    i--;
-	    if (i == size_t(-1))
-		break;
 	}
 	if (i == size_t(-1))
 	    break;
