@@ -1,8 +1,8 @@
-/*  $Id: BoolExpr.h,v 1.19 2008/10/08 04:19:32 sarrazip Exp $
+/*  $Id: BoolExpr.h,v 1.23 2013/03/03 08:19:28 sarrazip Exp $
     BoolExpr.h - Boolean expression binary tree node
 
     boolstuff - Disjunctive Normal Form boolean expression library
-    Copyright (C) 2002-2005 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2002-2013 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #ifndef _H_BoolExpr
@@ -100,8 +100,8 @@ public:
 	    delete root;<BR>
 
 	@param	t	type of the node (must be AND, OR or NOT)
-	@param	l	subtree to attach as the left-hand child (may be NULL)
-	@param	r	subtree to attach as the right-hand child (may be NULL)
+	@param	l	subtree to attach as the left-hand child (NULL for NOT)
+	@param	r	subtree to attach as the right-hand child
     */
     BoolExpr(Type t, BoolExpr *l, BoolExpr *r);
 
@@ -218,11 +218,15 @@ public:
 	@returns	the root of the transformed tree (may be NULL)
     */
     static BoolExpr *getDisjunctiveNormalForm(BoolExpr *root);
- 
+
     /**
 	Like getDisjunctiveNormalForm(), but without simplifications.
+	@param	root	  root of the tree to transform
+        @param  tooLarge  indicates if the resulting expression would be too large;
+                          check this when the returned pointer is null.
+	@returns	  the root of the transformed tree (may be NULL)
     */
-    static BoolExpr *getRawDNF(BoolExpr *root);
+    static BoolExpr *getRawDNF(BoolExpr *root, bool &tooLarge);
 
     /**
 	Determines if the tree rooted at this node is in the DNF.
@@ -328,13 +332,22 @@ private:
 
     friend class BoolExprParser;
 
+    static void simplifyConjunctionList(std::vector<BoolExpr<T> *> &conjunctionList);
+    static BoolExpr<T> *simplifyConjunction(BoolExpr<T> *conj);
     static void simplifyXAndXTerms(std::vector<BoolExpr<T> *> &terms);
-    static void destroyDNFOrNodes(BoolExpr<T> *root);
+    static void destroyDNFBinaryOpNodes(BoolExpr<T> *root, bool destroyOrNodes);
     static BoolExpr<T> *joinTreesWithOrNodes(
 				const std::vector<BoolExpr<T> *> &trees);
     bool isDNFTermUseful(const std::set<T> &positives,
 			 const std::set<T> &negatives) const;
-    static BoolExpr<T> *negateDNF(BoolExpr<T> *root);
+    static BoolExpr<T> *negateDNF(BoolExpr<T> *root, bool &tooLarge);
+
+    template <class OutputIter>
+    OutputIter getDNFFactorRoots(OutputIter dest) const;
+
+    // Forbidden operations:
+    BoolExpr<T>(const BoolExpr<T> &);
+    BoolExpr<T> &operator = (const BoolExpr<T> &);
 };
 
 
