@@ -1,4 +1,4 @@
-/*  $Id: checkdnf.cpp,v 1.13 2007/08/26 18:45:26 sarrazip Exp $
+/*  $Id: checkdnf.cpp,v 1.15 2013/03/03 06:48:45 sarrazip Exp $
     checkdnf.cpp - C++ API test program
 
     boolstuff - Disjunctive Normal Form boolean expression library
@@ -16,8 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 /*
@@ -34,7 +34,7 @@ using namespace boolstuff;
 
 
 int
-main(int argc, char *argv[])
+main(int /* argc */, char * /*argv*/ [])
 {
     BoolExprParser parser;
     string line;
